#!/usr/bin/env python3
# -*- coding: utf-8 -*-
# Copyright © 2015 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

# Launch Twitterwatch
'''Launch Twitterwatch'''

import sys
from twitterwatch.main import Main


class Twitterwatch(object):
    '''Twitterwatch class'''
    def __init__(self):
        '''Constructor of the Twitterwatch class'''
        self.main()

    def main(self):
        '''main of the Main class'''
        Main()

if __name__ == '__main__':
    Main()
    sys.exit(0)



# -*- coding: utf-8 -*-
# Copyright © 2015 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Main class
'''Main class'''

import configparser
import datetime
import os.path
import sys
import tweepy

from twitterwatch.cliparse import CliParse
from twitterwatch.confparse import ConfParse
from twitterwatch.emailwarning import EmailWarning

class Main(object):
    '''Main class'''
    def __init__(self):
        '''Constructor of the Main class'''
        # parse the command line
        rtargs = CliParse()
        pathtoconf = rtargs.configfile
        # read the configuration file
        cfgparse = ConfParse(pathtoconf)
        self.cfgvalues = cfgparse.confvalues

        # activate the twitter api
        self.auth = tweepy.OAuthHandler(self.cfgvalues['consumer_key'],
                                        self.cfgvalues['consumer_secret'])
        self.auth.secure = True
        self.auth.set_access_token(self.cfgvalues['access_token'],
                                    self.cfgvalues['access_token_secret'])
        self.api = tweepy.API(self.auth, wait_on_rate_limit=True, wait_on_rate_limit_notify=True)
        self.main()

    def main(self):
        '''Main of the Main class'''
        # get the 20 last tweets
        lasttweets = self.api.user_timeline()
        # see if the last tweet of twitter api was sent already
        lasttweet = lasttweets[0]
        # find the date of the last tweet
        lastactiondate = lasttweet.created_at
        # get the current date
        currentdate = datetime.datetime.now()
        # get the interval between two checks
        pause = datetime.timedelta(minutes=self.cfgvalues['check_interval'])
        if (currentdate - pause) > lastactiondate:
            user = self.api.me().screen_name
            # warn
            warning = 'Twitter stream of {} has stopped since {}'.format(user, lastactiondate.strftime("%d/%m/%Y - %H:%M:%S")) 
            emailwarn = EmailWarning(self.cfgvalues, warning, user)
            print(warning)



# -*- coding: utf-8 -*-
# Copyright © 2015 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# Get values of the configuration file
'''Get values of the configuration file'''

import configparser
import sys

class ConfParse(object):
    '''ConfParse class'''
    def __init__(self, pathtoconf):
        '''Constructor of the ConfParse class'''
        self.consumer_key = ''
        self.consumer_secret = ''
        self.check_interval = 60
        self.pathtoconf = pathtoconf
        self.mailhost = ''
        self.mailfrom = ''
        self.mailto = ''
        self.main()

    def main(self):
        '''Main of the ConfParse class'''
        # read the configuration file
        config = configparser.ConfigParser()
        try:
            with open(self.pathtoconf) as conffile:
                config.read_file(conffile)
                section='twitter'
                if config.has_section(section):
                    self.consumer_key = config.get(section, 'consumer_key')
                    self.consumer_secret = config.get(section, 'consumer_secret')
                    self.access_token = config.get(section, 'access_token')
                    self.access_token_secret = config.get(section, 'access_token_secret')
                section='schedule'
                if config.has_section(section):
                    self.check_interval= config.get(section, 'check_interval')
                section='mail'
                if config.has_section(section):
                    self.mailhost = config.get(section, 'host')
                    self.mailfrom = config.get(section, 'from')
                    self.mailto = config.get(section, 'to')

        except (configparser.Error, IOError, OSError) as err:
            print(err)
            sys.exit(1)
        try:
            self.check_interval = int(self.check_interval)
        except ValueError as err:
            print(err)
            self.check_interval = 60

    @property
    def confvalues(self):
        '''get the values of the configuration file'''
        return {'consumer_key': self.consumer_key,
                'consumer_secret': self.consumer_secret,
                'access_token': self.access_token,
                'access_token_secret': self.access_token_secret,
                'check_interval': self.check_interval,
                'mailhost': self.mailhost,
                'mailfrom': self.mailfrom,
                'mailto': self.mailto}



# -*- coding: utf-8 -*-
# Copyright © 2015 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/

# EmailWarning class
'''EmailWarning class'''

import smtplib
from email.mime.text import MIMEText

class EmailWarning(object):
    '''EmailWarning class'''
    def __init__(self, cfgvalues, warning, user):
        '''Constructor of the EmailWarning class'''
        self.cfgvalues = cfgvalues
        self.warning = warning
        self.user = user
        self.main()

    def main(self):
        '''Main of the Main class'''
        try:
            msg = MIMEText(self.warning)
            msg['Subject'] = 'Twitter stream of {} has stopped'.format(self.user)
            msg['From'] = self.cfgvalues['mailfrom']
            msg['To'] = self.cfgvalues['mailto']
            s = smtplib.SMTP(self.cfgvalues['mailhost'])
            s.send_message(msg)
            s.quit()
        except smtplib.SMTPException as err:
            print('smtplib error: {}'.format(err))



# -*- coding: utf-8 -*-
# Copyright © 2015 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

# CLI parsing
'''CLI parsing'''

import os.path
import sys

class CliParse(object):
    '''CliParse class'''
    def __init__(self):
        '''Constructor for the CliParse class'''
        self.pathtoconf = sys.argv[-1]
        self.main()

    def main(self):
        '''main of CliParse class'''
        # checks for the path to the configuration
        if self.pathtoconf.endswith('twitterwatch.py') or self.pathtoconf.endswith('twitterwatch'):
            print('No config file was provided. Exiting.')
            sys.exit(0)
        if not os.path.exists(self.pathtoconf):
            print('the path you provided for yaspe configuration file does not exists')
            sys.exit(1)
        if not os.path.isfile(self.pathtoconf):
            print('the path you provided for yaspe configuration is not a file')
            sys.exit(1)
        self.validpathtoconf = self.pathtoconf

    @property
    def configfile(self):
        '''return the path to the config file'''
        return self.validpathtoconf



# -*- coding: utf-8 -*-
# Copyright © 2015 Carl Chenet <carl.chenet@ohmytux.com>
# This program is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program.  If not, see <http://www.gnu.org/licenses/>

# Setup for Twitterwatch
'''Setup for Twitterwatch'''

import os.path

from setuptools import setup

CLASSIFIERS = [
    'Intended Audience :: End Users/Desktop',
    'Environment :: Console',
    'License :: OSI Approved :: GNU General Public License (GPL)',
    'Operating System :: POSIX :: Linux',
    'Programming Language :: Python :: 3.4'
]

setup(
    name='twitterwatch',
    version='0.1',
    license='GNU GPL v3',
    description='Twitter bot to watch a Twitter user timeline',
    long_description='Twitterwatch watches a user timeline in order to check if tweets are tweeted on a regular schedule.',
    classifiers=CLASSIFIERS,
    author='Carl Chenet',
    author_email='chaica@ohmytux.com',
    url='https://github.com/chaica/twitterwatch',
    download_url='https://github.com/chaica/twitterwatch',
    packages=['twitterwatch'],
    scripts=['scripts/twitterwatch'],
    install_requires=['tweepy>=3.3.0'],
)
